﻿
using System;
using System.Collections.Generic;
using Telerik.Windows.Controls.ScheduleView;
using Telerik.Windows.Controls;
using System.ComponentModel;
namespace GroupingAdvancedExample
{
	class ViewModel : ViewModelBase
	{
		private bool showBusinessLayer = true;
		private bool showDataLayer = true;
		private bool showPresentationLayer = true;
		private bool enableGrouping = true;
		private GroupDescriptionCollection groupDescriptions;
		private Func<object, bool> groupFilter;

		public bool ShowBusinessLayer
		{
			get
			{
				return this.showBusinessLayer;
			}
			set
			{
				if (this.showBusinessLayer != value)
				{
					this.showBusinessLayer = value;
					this.OnPropertyChanged(() => this.ShowBusinessLayer);
					this.UpdateGroupFilter();
				}
			}
		}

		public bool ShowDataLayer
		{
			get
			{
				return this.showDataLayer;
			}
			set
			{
				if (this.showDataLayer != value)
				{
					this.showDataLayer = value;
					this.OnPropertyChanged(() => this.ShowDataLayer);
					this.UpdateGroupFilter();
				}
			}
		}

		public bool ShowPresentationLayer
		{
			get
			{
				return this.showPresentationLayer;
			}
			set
			{
				if (this.showPresentationLayer != value)
				{
					this.showPresentationLayer = value;
					this.OnPropertyChanged(() => this.ShowPresentationLayer);
					this.UpdateGroupFilter();
				}
			}
		}

		public bool EnableGrouping
		{
			get
			{
				return this.enableGrouping;
			}
			set
			{
				if (this.enableGrouping != value)
				{
					this.enableGrouping = value;
					this.OnPropertyChanged(() => this.EnableGrouping);

					this.UpdateGroupDescriptions();
				}
			}
		}

		public GroupDescriptionCollection GroupDescriptions
		{
			get
			{
				if (this.groupDescriptions == null)
				{
					this.groupDescriptions = new GroupDescriptionCollection() { new DateGroupDescription() };
					this.UpdateGroupDescriptions();
				}
				return this.groupDescriptions;
			}
		}

		public Func<object, bool> GroupFilter
		{
			get
			{
				return this.groupFilter;
			}
			private set
			{
				this.groupFilter = value;
				this.OnPropertyChanged(() => this.GroupFilter);
			}
		}

		private void UpdateGroupFilter()
		{
			this.GroupFilter = new Func<object, bool>(this.GroupFilterFunc);
		}

		private bool GroupFilterFunc(object groupName)
		{
			IResource resource = groupName as IResource;

			if (resource == null)
				return true;

			IEnumerable<string> enumerable = GetEnabledGroups();

			foreach (string str in enumerable)
			{
				if (str.ToLower().Equals(resource.ResourceName.ToLower()))
					return true;
			}

			return false;
		}

		private ObservableAppointmentCollection appointments;
		public ObservableAppointmentCollection Appointments
		{
			get
			{
				return this.appointments;
			}
			set
			{
				this.appointments = value;
			}
		}

		public ViewModel()
		{
			appointments = new ObservableAppointmentCollection();

			Appointment appointment = new Appointment();
			appointment.Start = DateTime.Now;
			appointment.End = DateTime.Now.AddHours(1);
			appointment.Subject = "New appointment";
			Resource r = new Resource();
			r.ResourceName = "Periodical";
			r.ResourceType = "Category";
			appointment.Resources.Add(r);
			Appointments.Add(appointment);
		}

		private IEnumerable<string> GetEnabledGroups()
		{
			List<string> enabledGroups = new List<string>();

			if (this.ShowPresentationLayer) enabledGroups.Add("Fixed");
			if (this.ShowDataLayer) enabledGroups.Add("PeriodicFixed");
			if (this.ShowBusinessLayer) enabledGroups.Add("Periodical");

			return enabledGroups;
		}

		private void UpdateGroupDescriptions()
		{
			if (this.EnableGrouping)
			{
				ResourceGroupDescription groupDescription = new ResourceGroupDescription();
				groupDescription.ResourceType = "Category";
				this.GroupDescriptions.Add(groupDescription);
			}
			else
			{
				this.GroupDescriptions.RemoveAll((GroupDescription g) => g is ResourceGroupDescription);
			}
		}
	}
}